#!/usr/bin/env python
# encoding: utf-8
"""
Tests for parsing the result HTML.
"""

from astropy.utils.data import get_pkg_data_filename

from ..cmd import CmdResponseParser


def test_parse():
    """Test CmdResponseParser for getting .dat results from reponse HTML."""
    truth = "http://stev.oapd.inaf.it/~lgirardi/tmp/output657220915964.dat"
    htmlpath = get_pkg_data_filename("../data/cmd_isoc_output.html")
    with open(htmlpath, 'r') as f:
        html = f.read()
        p = CmdResponseParser()
        p.feed(html)
        url = p.data_url
    assert url == truth



#!/usr/bin/env python
# encoding: utf-8
"""
resultcache manages the cache of results from the CMD and TRILGEGAL
interfaces. To be used internally.
"""

import os


class PadovaCache(object):
    """Cache manager for CMD and TRILEGAL requests.

    The cache takes :class:`padova.settings.Settings` instances to hash
    results in the cache.
    """
    def __init__(self):
        super(PadovaCache, self).__init__()
        self._dir = os.path.expanduser("~/.padova_cache")
        if not os.path.exists(self._dir):
            os.makedirs(self._dir)

    def _cache_path(self, settings):
        return os.path.join(self._dir, str(settings.__hash__()))

    def __contains__(self, settings):
        return os.path.exists(self._cache_path(settings))

    def __getitem__(self, settings):
        assert self.__contains__(settings)
        with open(self._cache_path(settings)) as f:
            data = f.read()
        return data

    def __setitem__(self, settings, data):
        p = self._cache_path(settings)
        if os.path.exists(p):
            os.remove(p)
        with open(p, 'w') as f:
            f.write(data)



#!/usr/bin/env python
# encoding: utf-8
"""
Utilities
"""


def compression_type(filename, stream=False):
    """ Detect potential compressed file
    Returns the gz, bz2 or zip if a compression is detected, else None.

    From ezpadova by Morgan Fousneau
    """
    magic_dict = {"\x1f\x8b\x08": "gz",
                  "\x42\x5a\x68": "bz2",
                  "\x50\x4b\x03\x04": "zip"}

    max_len = max(len(x) for x in magic_dict)
    if not stream:
        with open(filename) as f:
            file_start = f.read(max_len)
        for magic, filetype in magic_dict.items():
            if file_start.startswith(magic):
                return filetype
    else:
        for magic, filetype in magic_dict.items():
            if filename[:len(magic)] == magic:
                return filetype

    return None



#!/usr/bin/env python
# encoding: utf-8
"""
Tests from the :mod:`lfdata` module.
"""

from astropy.utils.data import get_pkg_data_filename

from ..lfdata import LFTable


def test_read_lfs():
    """Test reading an isochrone table."""
    datpath = get_pkg_data_filename('../data/lf_0019.dat')
    print datpath
    lftable = LFTable(datpath)
    assert len(lftable.lfs) == 36

def test_lf():
    """Test a luminosity function instance."""
    datpath = get_pkg_data_filename('../data/lf_0019.dat')
    print datpath
    lftable = LFTable(datpath)
    lf = lftable.lfs[0]
    assert isinstance(lf.z, float)
    assert lf.z == 0.019
    assert isinstance(lf.info, list)



#!/usr/bin/env python
# encoding: utf-8
"""
Tools for reading CMD-interface tables outputs.
"""

from collections import deque


class BaseReader(object):
    """Baseclass for reading tables produced by the Padova CMD interface."""
    def __init__(self, f):
        super(BaseReader, self).__init__()
        self._f = f  # file handle
        self._read()

    def _prescan_table(self, n_header_lines):
        """Find lines where individual isochrones start."""
        header_lines = []

        data_blocks = []
        hdeque = deque()
        self._f.seek(0)
        block = None
        for i, line in enumerate(self._f):
            if line.startswith('#'):
                # This is a header line
                if block is None:
                    block = {}
                    if len(data_blocks) > 0:
                        data_blocks[-1]['end'] = i - 1
                hdeque.append(line.lstrip('#').strip())
            elif block is not None:
                # This is a data line;
                # Close the block
                block['start'] = i
                block['header_lines'] = [hdeque.pop()
                                         for j in xrange(n_header_lines)][::-1]
                data_blocks.append(block)

                # Treat excess header lines as a global header
                while len(hdeque) > 0:
                    header_lines.append(hdeque.popleft())

                block = None
        data_blocks[-1]['end'] = i
        n_lines = i + 1

        return header_lines, data_blocks, n_lines



# encoding: utf-8

# see http://zestreleaser.readthedocs.org/en/latest/versions.html
import pkg_resources
__version__ = pkg_resources.get_distribution("padova").version

from padova.cmd import IsochroneRequest  # NOQA
from padova.cmd import AgeGridRequest  # NOQA
from padova.cmd import MetallicityGridRequest  # NOQA



#!/usr/bin/env python
# encoding: utf-8
"""
This :mod:`isocdata` module helps to read Padova isochrone table files,
and split them into individual isochrones.
"""

import os
from collections import OrderedDict

import numpy as np
from astropy.table import Table

from padova.basereader import BaseReader


class IsochroneSet(BaseReader):
    """Reads an isochrone table (output from the Padova CMD interface).

    Parameters
    ----------
    fname :
        File handle of dataset to read
    """
    def __init__(self, f):
        self._isochrones = []
        super(IsochroneSet, self).__init__(f)
        self._current = 0

    def __iter__(self):
        return self

    def next(self):
        try:
            isoc = self._isochrones[self._current]
            self._current += 1
        except IndexError:
            self._current = 0
            raise StopIteration
        return isoc

    def __getitem__(self, index):
        return self._isochrones[index]

    def __len__(self):
        return len(self._isochrones)

    @property
    def isochrones(self):
        return self._isochrones

    def _read(self):
        """Read isochrone table and create Isochrone instances."""
        self._isochrones = []
        self._header_lines, blocks, n_lines = self._prescan_table(2)

        # Load the entire dataset
        colnames = self._parse_colnames(blocks[0]['header_lines'][-1])
        # skip first column because it's a blank tab
        usecols = [i + 1 for i, c in enumerate(colnames)]
        dt = []
        for cname in colnames:
            if cname == 'stage':
                dt.append((cname, 'S40'))
            elif cname == 'pmode':
                dt.append((cname, np.int))
            else:
                dt.append((cname, np.float))
        self._f.seek(0)
        data = np.genfromtxt(self._f,
                             dtype=np.dtype(dt),
                             delimiter='\t',
                             autostrip=True,
                             comments='#',
                             usecols=usecols)

        # Find where age or metallicity changed
        age_diffs = np.diff(data['logageyr'])
        z_diffs = np.diff(data['Z'])
        changed = np.where((age_diffs >= 0.0001) | (z_diffs >= 0.000001))[0]
        changed += 1  # needed to get indexes right below
        assert len(changed) + 1 == len(blocks)

        # Do this so we can iterate all blocks uniformly
        changed = np.concatenate([[0], changed, [len(data) + 1]])

        # Build individual isochrone tables
        for i, (idx0, block) in enumerate(zip(changed[:-1], blocks)):
            idx1 = changed[i + 1]
            isoc_data = data[idx0:idx1]
            meta = self._parse_meta(block['header_lines'][0])
            meta['header'] = self._header_lines
            tbl = Isochrone(isoc_data, meta=meta)
            isoc = Isochrone(tbl)
            self._isochrones.append(isoc)

    def _parse_colnames(self, header):
        header = header.replace('\t', ' ')
        parts = header.split()
        return parts

    def _parse_meta(self, header):
        header = header.replace('\t', ' ')
        header = header.replace('=', ' ')
        parts = header.split()[1:-1]
        vals = [float(p) for p in parts[1::2]]
        meta = OrderedDict(zip(parts[::2], vals))
        return meta


class Isochrone(Table):
    """Holds a single isochrone (single age, and metallicity).

    An :class:`Isochrone` is a :class:`astropy.table.Table`, but with
    additional attributes and methods.
    """
    def __init__(self, *args, **kwargs):
        super(Isochrone, self).__init__(*args, **kwargs)

    @property
    def z(self):
        """Metallicity of this isochrone."""
        return self.meta['Z']

    @property
    def age(self):
        """Age of this isochrone (yr)."""
        return self.meta['Age']

    @property
    def z_code(self):
        """Code string for the metallicity. This is a 4-digit code for the
        metallicity (with leading zeros). E.g. ``z=0.012`` will be ``0120``.
        """
        z_str = "%.4f" % self.z
        z_str = z_str[2:]
        return z_str

    @property
    def age_code(self):
        """Code string for the age. This is in format ``tt.tt`, giving the
        log age.
        """
        return "%05.2f" % np.log10(self.age)

    @property
    def info(self):
        """Info about how this isochrone was generated
        (header from CMD output).
        """
        return self.meta['header']

    @property
    def filter_names(self):
        """A list of (column) names of the filters included in this isochrone.
        """
        non_mag_cnames = ['log(age/yr)', 'M_ini', 'M_act', 'logL/Lo', 'logTe',
                          'logG', 'mbol', 'C/O', 'M_hec', 'period', 'pmode',
                          'logMdot',
                          'int_IMF', 'stage',
                          'Z', 'logageyr', 'logLLo']
        # Normally I'd use sets, but column ordering is important
        return [name for name in self.colnames
                if name not in non_mag_cnames]

    def export_for_starfish(self, output_dir, bands=None):
        """Export the isochrone in a format useful for StarFISH `mklib`.

        Parameters
        ----------
        output_dir : str
            Directory where the isochrone file will be saved. The full
            filename is generated by the method to conform to the standard
            naming scheme: ``<output_dir>/zNNNN_tt.tt.dat`` where ``NNNN``
            is a 4-digit code for the metallicity (with leading zeros) and
            ``tt.tt`` is the log of the age.
        bands : list
            List of bands to include in isochrone output. Only necessary for
            exporting a subset of the bands in the full isochrone dataset.
        """
        filename = "z%s_%s" % (self.z_code, self.age_code)
        output_path = os.path.join(output_dir, filename)
        if not os.path.exists(output_dir):
            os.makedirs(output_dir)
        if os.path.exists(output_path):
            os.remove(output_path)
        fmt = {'m_ini': "%10.7f"}  # table writer formatting
        if bands is None:
            bandnames = self.filter_names
            for name in bandnames:
                fmt[name] = "%8.6f"
        else:
            bandnames = [n for n in bands if n in self.filter_names]
        self.write(output_path,
                   format='ascii.fixed_width_no_header',
                   formats=fmt,
                   delimiter=' ',
                   delimiter_pad=None,
                   bookend=False,
                   include_names=['M_ini'] + bandnames)



#!/usr/bin/env python
# encoding: utf-8
"""
Maps for cmd interface settings

Code adapted from ezpadova - Morgan Fousneau, MIT License
"""

from __future__ import print_function, unicode_literals, division

import sys

if sys.version_info[0] > 2:
    py3k = True
    from urllib.parse import urlencode
    from urllib import request
    from urllib.request import urlopen
    from io import StringIO
    from html import parser
else:
    py3k = False
    from urllib import urlencode
    from urllib2 import urlopen
    from StringIO import StringIO
    import HTMLParser as parser

import zlib
import re

from padova.resultcache import PadovaCache
from padova.utils import compression_type
from padova.isocdata import IsochroneSet


class CMDRequest(object):
    """Python interface to the Padova group's CMD web interface for isochrones.

    Parameters
    ----------
    settings : :class:`padova.settings.Settings`
        A settings instance loaded with user settings.
    """
    def __init__(self, settings):
        super(CMDRequest, self).__init__()
        self._cache = PadovaCache()
        self.settings = settings
        if self.settings in self._cache:
            # Get request from the cache
            # print("Reading from cache")
            self._r = self._cache[self.settings]
        else:
            # Call API and cache it
            self._r = self._request()
            self._cache[self.settings] = self._r
        self._isochrone_set = None

    def _request(self):
        """Request isochromes from CMD."""
        webserver = 'http://stev.oapd.inaf.it'
        # FIXME convert to log
        # print('Requesting from {0}...'.format(webserver))
        url = webserver + '/cgi-bin/cmd'
        q = urlencode(self.settings.settings)
        if py3k:
            req = request.Request(url, q.encode('utf8'))
            c = urlopen(req).read().decode('utf8')
        else:
            c = urlopen(url, q).read()
        # Find the output dataset URL in the HTML that CMD returns
        aa = re.compile('output\d+')
        fname = aa.findall(c)
        if len(fname) > 0:
            url = '{0}/~lgirardi/tmp/{1}.dat'.format(webserver, fname[0])
            # FIXME convert to log
            # print('Downloading data...{0}'.format(url))
            bf = urlopen(url)
            r = bf.read()
            # Decompress the data if necessary
            typ = compression_type(r, stream=True)
            if typ is not None:
                r = zlib.decompress(bytes(r), 15 + 32)
            return r
        else:
            # print(c)
            print(url + q)
            if "errorwarning" in c:
                p = CMDErrorParser()
                p.feed(c)
                print('\n', '\n'.join(p.data).strip())
            raise RuntimeError('Server Response is incorrect')

    @property
    def isochrone_set(self):
        """IsochroneSet table with the isochrones."""
        if self._isochrone_set is None:
            f = StringIO(self._r)
            self._isochrone_set = IsochroneSet(f)
        return self._isochrone_set

    @property
    def data(self):
        return self._r


class CMDErrorParser(parser.HTMLParser):
    """Find error box in the recent version of CMD website."""
    def handle_starttag(self, tag, attrs):
        if (tag == "p") & (dict(attrs).get('class', None) == 'errorwarning'):
            self._record = True
            self.data = []

    def handle_endtag(self, tag):
        if (tag == "p") & getattr(self, '_record', False):
            self._record = False

    def handle_data(self, data):
        if getattr(self, '_record', False):
            self.data.append(data)



#!/usr/bin/env python
# encoding: utf-8
"""
Python interface to the Padova group's CMD web interface for isochrones.
"""

from __future__ import print_function, unicode_literals, division

from padova.settings import Settings
from padova.interface import CMDRequest


class IsochroneRequest(CMDRequest):
    """Request a single isochrone.

    Parameters
    ----------
    z : float
        The metallicity, fraction of metals in stellar composition.
    log_age : float
        The age, :math:`\log_{10} (A/\mathrm{yr})`.
    kwargs :
        Keyword arguments, see TODO
    """
    def __init__(self, z=0.0, log_age=9., **kwargs):
        kwargs['isoc_val'] = "0"  # declare single isochrone request
        kwargs['isoc_zeta'] = z
        kwargs['isoc_age'] = 10. ** log_age
        s = Settings.load_package_settings(**kwargs)
        super(IsochroneRequest, self).__init__(s)

    @property
    def isochrone(self):
        """The :class:`padova.isocdata.Isochrone` instance, an Astropy Table.
        """
        return self.isochrone_set.isochrones[0]


class AgeGridRequest(CMDRequest):
    """Request a grid of single-Z isochrones spanning an age range.

    Parameters
    ----------
    z : float
        The metallicity, fraction of metals in stellar composition.
    min_log_age : float
        Youngest isochrone, :math:`\log_{10} (A/\mathrm{yr})`.
    max_log_age : float
        Oldest isochrone, :math:`\log_{10} (A/\mathrm{yr})`.
    delta_log_age : float
        Isochrone age step size, in log-years.
    kwargs :
        Keyword arguments, see TODO
    """
    def __init__(self, z=0.0,
                 min_log_age=6.6, max_log_age=10.13, delta_log_age=0.05,
                 **kwargs):
        kwargs['isoc_val'] = "1"  # declare age grid request
        kwargs['isoc_lage0'] = min_log_age
        kwargs['isoc_lage1'] = max_log_age
        kwargs['isoc_dlage'] = delta_log_age
        kwargs['isoc_zeta0'] = z
        s = Settings.load_package_settings(**kwargs)
        super(AgeGridRequest, self).__init__(s)


class MetallicityGridRequest(CMDRequest):
    """Request a grid of single-age isochrones spanning a metallicity range.

    Parameters
    ----------
    log_age : float
        The age, :math:`\log_{10} (A/\mathrm{yr})`.
    min_z : float
        The minimum metallicity of the grid (fraction of composition).
    max_z : float
        The maximum metallicity of the grid (fraction of composition).
    delta_z : float
        The metallicity step size (fraction of composition).
    kwargs :
        Keyword arguments, see TODO
    """
    def __init__(self, log_age=9.,
                 min_z=0.0001, max_z=0.03, delta_z=0.0001,
                 **kwargs):
        kwargs['isoc_val'] = "2"  # declare Z grid request
        kwargs['isoc_age0'] = 10. ** log_age
        kwargs['isoc_z0'] = min_z
        kwargs['isoc_z1'] = max_z
        kwargs['isoc_dz'] = delta_z
        s = Settings.load_package_settings(**kwargs)
        super(MetallicityGridRequest, self).__init__(s)



#!/usr/bin/env python
# encoding: utf-8
"""
Read/represent luminosity function tables.
"""

import linecache

import numpy as np
from astropy.table import Table

from .basereader import BaseReader


class LFTable(BaseReader):
    """Read a luminosity function table (output from the Padova CMD interface).

    Attributes
    ----------
    metadata : list
        List of lines from the metadata found at the top of the ischrone
        table file.
    lfs : list
        List of :class:`LuminosityFunction` instances in the table.
    
    Parameters
    ----------

    fname : str
        Filename of LF table to read (either a .dat or a .dat.gz).
    """
    def __init__(self, fname):
        super(LFTable, self).__init__(fname)

    def _read(self):
        """Read isochrone table and create LuminosityFunction instances."""
        start_indices = self._prescan_table()
        print "start_indices",
        print start_indices
        self._lf_specs = self._read_lf_specs(start_indices)
        self.metadata = self._read_metadata(0, start_indices[0])
        self.lfs = self._read_lfs(start_indices)
        linecache.clearcache()

    def _read_lfs(self, start_indices):
        """Extract isochrones from the table, creating individual
        :class:`Isochrone` instances.
        """
        lfs = []
        for i, (start_index, meta) in enumerate(
                zip(start_indices, self._lf_specs)):
            if i < len(start_indices) - 1:
                end_index = start_indices[i + 1]
            else:
                end_index = self._linecount()
            isoc = self._read_lf(start_index, end_index, meta)
            lfs.append(isoc)
        return lfs

    def _read_lf_specs(self, start_indices):
        """Produce a list of the age and metallicity specifications for
        each isochone in the table.
        """
        specs = []
        for i in start_indices:
            line = linecache.getline(self.fname, i + 1).rstrip().split()
            z = float(line[7])
            age = float(line[10])
            specs.append({"Z": z, "age": age})
        return specs

    def _read_lf(self, start_index, end_index, meta):
        """Read a single LF, between `start_index` and `end_index`."""
        header = self._read_header(start_index)
        # build datatype for structured array
        dt = [(cname, np.float) for cname in header]
        data = np.empty(end_index - start_index - 2, dtype=np.dtype(dt))
        for j, i in enumerate(xrange(start_index + 2, end_index)):
            parts = linecache.getline(self.fname, i + 1).rstrip('\n').split()
            for val, (cname, typ) in zip(parts, dt):
                if typ == np.float:
                    data[cname][j] = float(val)
                elif typ == np.int:
                    data[cname][j] = int(val)
                else:
                    data[cname][j] = val
        tbl = Table(data,
                meta={"header": self.metadata,
                      "Z": meta['Z'],
                      'age': meta['age']})
        lf = LuminosityFunction(tbl)
        return lf


class LuminosityFunction(object):
    """Holds a single luminosity function (single age, metallicity) for
    several bandpasses.
    
    Parameters
    ----------

    table : :class:`astropy.table.Table` instance
        A table with luminosity function data. This table must also have
        metadata with age and metallicity information. This table is typically
        generated by :class:`LFTable`.
    """
    def __init__(self, table):
        super(LuminosityFunction, self).__init__()
        self.table = table

    @property
    def z(self):
        """Metallicity of this isochrone."""
        return self.table.meta['Z']

    @property
    def age(self):
        """Age of this isochrone (yr)."""
        return self.table.meta['age']

    @property
    def z_code(self):
        """Code string for the metallicity. This is a 4-digit code for the
        metallicity (with leading zeros). E.g. ``z=0.012`` will be ``0120``.
        """
        z_str = "%.4f" % self.z
        z_str = z_str[2:]
        return z_str

    @property
    def age_code(self):
        """Code string for the age. This is in format ``tt.tt`, giving the
        log age.
        """
        return "%05.2f" % np.log10(self.age)

    @property
    def info(self):
        """Info about how this isochrone was generated
        (header from CMD output).
        """
        return self.table.meta['header']

    @property
    def filter_names(self):
        """A list of (column) names of the filters included in this isochrone.
        """
        cnames = self.table.colnames
        non_mag_cnames = ['age/yr', 'bin(mag)', 'mbol']
        # Normally I'd use sets, but column ordering is important
        return [name for name in cnames if name not in non_mag_cnames]



#!/usr/bin/env python
# encoding: utf-8
"""
CMD Settings.

This module assists with documenting, validating and storing settings for
the CMD web api.
"""

import sys
import os
from collections import OrderedDict
import hashlib

if sys.version_info[0] > 2:
    from urllib.parse import urlencode
else:
    from urllib import urlencode

from pkg_resources import resource_stream
import pytoml as toml


class Settings(object):
    """Store user settings and validate against the schema for the Padova
    CMD web app.

    Parameters
    ----------
    f : stream-like
        A file stream to the TOML settings file.
    kwargs :
        User settings
    """
    def __init__(self, f, **kwargs):
        super(Settings, self).__init__()
        d = toml.load(f)
        # Ordering is import to consistently build a hash for caching
        self._schema = OrderedDict(sorted(d.items(), key=lambda t: t[0]))
        self._aliases = self._index_aliases()
        self._user_settings = {}
        # Self-validate
        self.validate()
        # Add any user settings
        self.update(kwargs)

    @classmethod
    def load_package_settings(cls, name="cmd_2_6.toml", **kwargs):
        """Load a settings file that ships with `Padova`.

        Parameters
        ----------
        name : str
            Name of the TOML file.
        kwargs :
            User settings
        """
        f = resource_stream(__name__,
                            os.path.join("data", "settings", name))
        return cls(f, **kwargs)

    def _index_aliases(self):
        """Build a hash of alias names back to full names."""
        aliases = {}
        for k, table in self._schema.iteritems():
            if 'alias' in table:
                aliases[table['alias']] = k
        return aliases

    def _resolve_key(self, k):
        if k in self._schema:
            key = k
        elif k in self._aliases:
            key = self._aliases[k]
        else:
            raise KeyError('Unknown settings key: {0}'.format(k))
        return key

    def _format_value(self, key, table, v):
        if 'format' in table:
            return table['format'].format(**{key: v})
        else:
            return v

    def _validate(self, key, v):
        k = self._resolve_key(key)
        table = self._schema[k]
        if table['kind'] == 'static':
            assert v == table['default'], 'Cannot override {0}'.format(key)
        elif table['kind'] == 'choices':
            assert v in table['choices'], '{0}: {1} invalid'.format(key, v)
        elif table['kind'] == 'range':
            r = table['range']
            assert (v >= min(r)) and (v <= max(r)), '{0}: {1} outside range'.\
                format(key, v)

    def validate(self):
        """Validates all settings: defaults or user overrides."""
        for k, v in self.iteritems():
            self._validate(k, v)

    def __setattr__(self, name, value):
        # Attempt to add a user's setting as an attribute
        try:
            k = self._resolve_key(name)
            self._validate(k, value)
            self._user_settings[k] = value
        except (AttributeError, KeyError):
            super(Settings, self).__setattr__(name, value)

    def __delattr__(self, name):
        # Attempt to delete a user's setting first
        try:
            k = self._resolve_key(name)
            if k in self._user_settings:
                del self._user_settings[k]
        except KeyError:
            super(Settings, self).__delattr__(name)

    def __len__(self):
        return len(self._schema)

    def __getitem__(self, key):
        k = self._resolve_key(key)
        try:
            return self._user_settings[k]
        except KeyError:
            return self._schema[k]['default']

    def __setitem__(self, key, value):
        k = self._resolve_key(key)
        self._validate(k, value)
        self._user_settings[k] = value

    def __delitem__(self, key):
        # Only delete user settings
        k = self._resolve_key(key)
        if k in self._user_settings:
            del self._user_settings[k]

    def __hash__(self):
        """Build a hash given the current settings."""
        # String to build hash against
        q = urlencode(self.settings)
        m = hashlib.md5()
        m.update(q)
        return m.hexdigest()

    @property
    def defaults(self):
        """A dict of the formatted default settings."""
        defs = OrderedDict()
        for k, table in self._schema.iteritems():
            key = self._resolve_key(k)
            v = table['default']
            val = self._format_value(key, table, v)
            defs[key] = val
        return defs

    @property
    def settings(self):
        """A dict of the formatted settings (including user settings)."""
        s = OrderedDict()
        for k, v in self.iteritems():
            table = self._schema[k]
            s[k] = self._format_value(k, table, v)
        return s

    def iteritems(self):
        """Iterate through all setting key-value pairs.

        Note: the values are *unformatted*.
        """
        for k, table in self._schema.iteritems():
            if k in self._user_settings:
                yield k, self._user_settings[k]
            else:
                yield k, table['default']

    def update(self, h):
        """Update the user settings with a dict-like."""
        for k, v in h.iteritems():
            key = self._resolve_key(k)
            self._validate(key, v)
            self._user_settings[key] = v



#!/usr/bin/env python
# encoding: utf-8
"""
Send a sample request to the CMD interface.

2013-11-29 - Created by Jonathan Sick
"""

from padova.cmd import CMD
from padova.isocdata import IsochroneTable


def main():
    cmd = CMD()
    cache_path = cmd.get()
    print cache_path
    tbl = IsochroneTable(cache_path)
    tbl.cleanup()
    print tbl.metadata


if __name__ == '__main__':
    main()



#!/usr/bin/env python
# encoding: utf-8

# Always prefer setuptools over distutils
from setuptools import setup, find_packages
# To use a consistent encoding
from codecs import open
from os import path

# Set affiliated package-specific settings
PACKAGENAME = 'padova'
DESCRIPTION = 'Helpers for using Padova isochrones.'
LONG_DESCRIPTION = ''
AUTHOR = 'Jonathan Sick'
AUTHOR_EMAIL = 'jonathansick@mac.com'
LICENSE = 'MIT'
URL = 'http://github.com/jonathansick/padova'

here = path.abspath(path.dirname(__file__))


# Get the long description from the relevant file
with open(path.join(here, 'README.rst'), encoding='utf-8') as f:
    long_description = f.read()


setup(
    name=PACKAGENAME,
    # VERSION should be PEP386 compatible
    # (http://www.python.org/dev/peps/pep-0386)
    # Versions should comply with PEP440.
    version='0.1',
    description=DESCRIPTION,
    long_description=long_description,
    url=URL,
    author='AUTHOR',
    author_email='AUTHOR_EMAIL',
    license='MIT',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Science/Research',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 2.7',
    ],
    keywords='astronomy stellarpopulations',
    packages=find_packages(exclude=['contrib', 'docs', 'tests*']),
    install_requires=['requests', 'numpy', 'astropy', 'pytoml'],

    package_data={
        'padova': ['padova/data/settings/*.toml'],
    },

    # To provide executable scripts, use entry points in preference to the
    # "scripts" keyword. Entry points provide cross-platform support and allow
    # pip to create the appropriate form of executable for the target platform.
    # entry_points={
    #     'console_scripts': [
    #         'sample=sample:main',
    #     ],
    # },
)



#!/usr/bin/env python
# encoding: utf-8
"""
Tests from the :mod:`isocdata` module.
"""

from astropy.utils.data import get_pkg_data_filename

from ..isocdata import IsochroneTable


def test_prescan():
    """Test the pre-scan done by IsochroneTable."""
    datpath = get_pkg_data_filename('../data/isocz0120.dat')
    print datpath
    isoctable = IsochroneTable(datpath)
    start_lines = isoctable._prescan_table()
    isoctable._read_isochrone_specs(start_lines)
    assert start_lines[0] == 11
    assert len(start_lines) == 71

def test_read_isochrones():
    """Test reading an isochrone table."""
    datpath = get_pkg_data_filename('../data/isocz0120.dat')
    print datpath
    isoctable = IsochroneTable(datpath)
    assert len(isoctable.isochrones) == 71

def test_isochrone():
    """Test an isochrone instance"""
    datpath = get_pkg_data_filename('../data/isocz0120.dat')
    print datpath
    isoctable = IsochroneTable(datpath)
    isoc = isoctable.isochrones[0]
    assert isinstance(isoc.z, float)
    assert isoc.z == 0.012
    assert isinstance(isoc.age, float)
    assert isoc.age == 3981000.0
    assert isinstance(isoc.info, list)

def test_isochrone_starfish():
    """Test the export-for-starfish feature of Isochrone."""
    datpath = get_pkg_data_filename('../data/isocz0120.dat')
    print datpath
    isoctable = IsochroneTable(datpath)
    isoc = isoctable.isochrones[0]
    for n, m in zip(isoc.filter_names, ['J', 'H', 'Ks']):
        assert n == m
    isoc.export_for_starfish("_test")
